@@ -78,8 +78,9 @@ class Issue(BaseResource):
     query_create = '/projects/{project_id}/issues.json'
     query_update = '/issues/{}.json'
     query_delete = '/issues/{}.json'
-    search_hints = ['issue', 'issue closed']
+    search_hints = ['issue', 'issue closed', 'issue-closed']
     extra_export_columns = ['description', 'last_notes']
+    manager_class = managers.IssueManager
 
     _repr = [['id', 'subject'], ['title'], ['id']]
     _includes = ['children', 'attachments', 'relations', 'changesets', 'journals', 'watchers', 'allowed_statuses']
@@ -174,6 +175,12 @@ class Issue(BaseResource):
 
         return super().decode(attr, value, manager)
 
+    def copy(self, link_original=True, include=(), **fields):
+        if 'project_id' not in fields and not self.is_new():
+            fields['project_id'] = self._decoded_attrs['project']['id']
+
+        return self.manager.copy(self.internal_id, link_original=link_original, include=include, **fields)
+
 
 class TimeEntry(BaseResource):
     redmine_version = (1, 1, 0)
@@ -402,7 +409,7 @@ class User(BaseResource):
     container_create = 'user'
     container_update = 'user'
     query_all_export = '/users.{format}'
-    query_all = '/users.json'
+    query_all = '/users.json?status='
     query_one = '/users/{}.json'
     query_filter = '/users.json'
     query_create = '/users.json'
@@ -412,7 +419,7 @@ class User(BaseResource):
 
     _repr = [['id', 'firstname', 'lastname'], ['id', 'name']]
     _includes = ['memberships', 'groups']
-    _relations = ['issues', 'time_entries']
+    _relations = ['issues', 'issues_assigned', 'issues_authored', 'time_entries']
     _relations_name = 'assigned_to'
     _unconvertible = ['status']
     _create_readonly = BaseResource._create_readonly + ['api_key', 'last_login_on']
@@ -422,12 +429,18 @@ class User(BaseResource):
         'groups': 'Group',
         'memberships': 'ProjectMembership',
         'issues': 'Issue',
+        'issues_assigned': 'Issue',
+        'issues_authored': 'Issue',
         'time_entries': 'TimeEntry',
     }
 
     def __getattr__(self, attr):
-        if attr == 'time_entries' and attr not in self._encoded_attrs:
-            self._relations_name = 'user'
+        if attr in self._relations and attr not in self._encoded_attrs:
+            if attr == 'issues_authored':
+                self._relations_name = 'author'
+            elif attr == 'time_entries':
+                self._relations_name = 'user'
+
             value = super().__getattr__(attr)
             self._relations_name = 'assigned_to'
             return value
