@@ -1,7 +1,7 @@
 from . import mock, BaseRedmineTestCase
 from .responses import responses
 
-from redminelib import resources, resultsets, exceptions
+from redminelib import resources, managers, resultsets, exceptions
 
 
 class StandardResourcesTestCase(BaseRedmineTestCase):
@@ -25,7 +25,7 @@ class StandardResourcesTestCase(BaseRedmineTestCase):
     def test_export(self):
         self.response.json.return_value = responses['issue']['get']
         self.response.iter_content = lambda chunk_size: (str(num) for num in range(0, 5))
-        self.assertEqual(self.redmine.issue.get(1).export('txt', '/foo/bar'), '/foo/bar/1.txt')
+        self.assertEqual(self.redmine.issue.get(1).export('txt', '/foo/bar/'), '/foo/bar/1.txt')
 
     def test_export_not_supported_exception(self):
         self.response.json.return_value = responses['attachment']['get']
@@ -183,12 +183,21 @@ class StandardResourcesTestCase(BaseRedmineTestCase):
         self.assertIn('subject', attributes)
         self.assertIn('relations', attributes)
         self.assertIn('time_entries', attributes)
+        self.assertIn('children', attributes)
+        self.assertIn('attachments', attributes)
+        self.assertIn('manager', attributes)
+        self.assertIn('url', attributes)
+        self.assertIn('internal_id', attributes)
 
     def test_supports_iteration(self):
         self.response.json.return_value = responses['project']['get']
-        project = list(self.redmine.project.get(1))
+        p = self.redmine.project.get(1)
+        project = list(p)
         self.assertIn(('name', 'Foo'), project)
         self.assertIn(('id', 1), project)
+        self.assertIn(('manager', p.manager), project)
+        self.assertIn(('url', f'{self.url}/projects/foo'), project)
+        self.assertIn(('internal_id', 1), project)
 
     def test_setting_custom_field_raises_exception_if_not_list_of_dicts(self):
         self.response.json.return_value = {'project': {'name': 'Foo', 'id': 1, 'custom_fields': [{'id': 1}]}}
@@ -288,7 +297,7 @@ class StandardResourcesTestCase(BaseRedmineTestCase):
         self.response.json.return_value = {
             'project': {'name': 'Foo', 'id': 1, 'custom_fields': [{'id': 1, 'value': 'foo'}]}}
         project = self.redmine.project.get(1)
-        project.homepage = 'http://foo.bar'
+        project.homepage = self.url
         project.parent_id = 3
         project.custom_fields = [{'id': 1, 'value': 'bar'}]
         self.assertIsInstance(project.save(), resources.Project)
@@ -345,7 +354,7 @@ class StandardResourcesTestCase(BaseRedmineTestCase):
     def test_project_export(self):
         self.response.json.return_value = responses['project']['all']
         self.response.iter_content = lambda chunk_size: (str(num) for num in range(0, 5))
-        self.assertEqual(self.redmine.project.all().export('txt', '/foo/bar'), '/foo/bar/projects.txt')
+        self.assertEqual(self.redmine.project.all().export('txt', '/foo/bar/'), '/foo/bar/projects.txt')
 
     def test_project_parent_converts_to_resource(self):
         self.response.json.return_value = {'project': {'name': 'Foo', 'id': 1, 'parent': {'id': 2}}}
@@ -363,6 +372,18 @@ class StandardResourcesTestCase(BaseRedmineTestCase):
         self.assertIsInstance(project.default_assignee, resources.User)
         self.assertEqual(project.default_assignee.id, 4)
 
+    def test_project_sets_attrs_from_single_attr_id_map(self):
+        self.response.json.return_value = responses['project']['get']
+        project = self.redmine.project.get(1)
+        project.parent_id = 1
+        self.assertEqual(project._decoded_attrs['parent'], {'id': 1})
+
+    def test_project_sets_attrs_from_multiple_attr_id_map(self):
+        self.response.json.return_value = responses['project']['get']
+        project = self.redmine.project.get(1)
+        project.tracker_ids = [1, 2]
+        self.assertEqual(project._decoded_attrs['trackers'], [{'id': 1}, {'id': 2}])
+
     def test_project_supports_close_reopen_archive_unarchive(self):
         self.response.json.return_value = responses['project']['get']
         project = self.redmine.project.get(1)
@@ -428,6 +449,32 @@ class StandardResourcesTestCase(BaseRedmineTestCase):
         self.assertIsInstance(issue.save(), resources.Issue)
         self.assertEqual(issue.custom_fields[0].value, 'bar')
 
+    def test_issue_copy(self):
+        import json
+        self.response.status_code = 201
+        self.response.json.return_value = {'issue': {'subject': 'Foo', 'id': 1, 'project': {'id': 1}}}
+        self.redmine.issue.get(1).copy()
+        request = json.loads(self.patch_requests.call_args[1]['data'])
+        self.assertEqual(request['copy_from'], 1)
+        self.assertEqual(request['link_copy'], '1')
+        self.assertEqual(request['copy_subtasks'], '1')
+        self.assertEqual(request['copy_attachments'], '1')
+
+    def test_issue_copy_via_manager(self):
+        import json
+        self.response.status_code = 201
+        self.response.json.return_value = responses['issue']['get']
+        self.redmine.issue.copy(1, project_id=1, link_original=False, include=None)
+        request = json.loads(self.patch_requests.call_args[1]['data'])
+        self.assertEqual(request['copy_from'], 1)
+        self.assertNotIn('link_copy', request)
+        self.assertNotIn('copy_subtasks', request)
+        self.assertNotIn('copy_attachments', request)
+
+    def test_issue_custom_manager(self):
+        self.assertEqual(repr(self.redmine.issue), '<redminelib.managers.IssueManager object for Issue resource>')
+        self.assertIsInstance(self.redmine.issue, managers.IssueManager)
+
     def test_issue_relations(self):
         self.response.json.return_value = responses['issue']['get']
         issue = self.redmine.issue.get(1)
@@ -561,9 +608,9 @@ class StandardResourcesTestCase(BaseRedmineTestCase):
     def test_issue_export(self):
         self.response.json.return_value = responses['issue']['all']
         self.response.iter_content = lambda chunk_size: (str(num) for num in range(0, 5))
-        self.assertEqual(self.redmine.issue.all().export('txt', '/foo/bar'), '/foo/bar/issues.txt')
+        self.assertEqual(self.redmine.issue.all().export('txt', '/foo/bar/'), '/foo/bar/issues.txt')
         self.response.json.return_value = responses['issue']['get']
-        self.assertEqual(self.redmine.issue.get(1).export('txt', '/foo/bar'), '/foo/bar/1.txt')
+        self.assertEqual(self.redmine.issue.get(1).export('txt', '/foo/bar/'), '/foo/bar/1.txt')
 
     def test_issue_parent_converts_to_resource(self):
         self.response.json.return_value = {'issue': {'subject': 'Foo', 'id': 1, 'parent': {'id': 2}}}
@@ -599,6 +646,32 @@ class StandardResourcesTestCase(BaseRedmineTestCase):
         self.assertIsInstance(issue.fixed_version, resources.Version)
         self.assertEqual(issue.fixed_version.id, 1)
 
+    def test_issue_sets_attrs_from_single_attr_id_map(self):
+        self.response.json.return_value = responses['issue']['get']
+        issue = self.redmine.issue.get(1)
+        issue.project_id = 1
+        issue.tracker_id = 1
+        issue.status_id = 1
+        issue.priority_id = 1
+        issue.category_id = 1
+        issue.fixed_version_id = 1
+        issue.assigned_to_id = 1
+        issue.parent_issue_id = 1
+        self.assertEqual(issue._decoded_attrs['project'], {'id': 1})
+        self.assertEqual(issue._decoded_attrs['tracker'], {'id': 1})
+        self.assertEqual(issue._decoded_attrs['status'], {'id': 1})
+        self.assertEqual(issue._decoded_attrs['priority'], {'id': 1})
+        self.assertEqual(issue._decoded_attrs['category'], {'id': 1})
+        self.assertEqual(issue._decoded_attrs['fixed_version'], {'id': 1})
+        self.assertEqual(issue._decoded_attrs['assigned_to'], {'id': 1})
+        self.assertEqual(issue._decoded_attrs['parent'], {'id': 1})
+
+    def test_issue_sets_attrs_from_multiple_attr_id_map(self):
+        self.response.json.return_value = responses['issue']['get']
+        issue = self.redmine.issue.get(1)
+        issue.watcher_user_ids = [1, 2]
+        self.assertEqual(issue._decoded_attrs['watchers'], [{'id': 1}, {'id': 2}])
+
     def test_time_entry_version(self):
         self.assertEqual(self.redmine.time_entry.resource_class.redmine_version, (1, 1, 0))
 
@@ -675,7 +748,7 @@ class StandardResourcesTestCase(BaseRedmineTestCase):
     def test_time_entry_export(self):
         self.response.json.return_value = responses['time_entry']['all']
         self.response.iter_content = lambda chunk_size: (str(num) for num in range(0, 5))
-        self.assertEqual(self.redmine.time_entry.all().export('txt', '/foo/bar'), '/foo/bar/time_entries.txt')
+        self.assertEqual(self.redmine.time_entry.all().export('txt', '/foo/bar/'), '/foo/bar/time_entries.txt')
 
     def test_time_entry_resource_map_converts_to_resource(self):
         self.response.json.return_value = responses['time_entry']['get']
@@ -693,6 +766,16 @@ class StandardResourcesTestCase(BaseRedmineTestCase):
         self.assertIsInstance(time_entry.activity, resources.Enumeration)
         self.assertEqual(time_entry.activity.id, 1)
 
+    def test_time_entry_sets_attrs_from_single_attr_id_map(self):
+        self.response.json.return_value = responses['time_entry']['get']
+        time_entry = self.redmine.time_entry.get(1)
+        time_entry.project_id = 1
+        time_entry.issue_id = 1
+        time_entry.activity_id = 1
+        self.assertEqual(time_entry._decoded_attrs['project'], {'id': 1})
+        self.assertEqual(time_entry._decoded_attrs['issue'], {'id': 1})
+        self.assertEqual(time_entry._decoded_attrs['activity'], {'id': 1})
+
     def test_enumeration_version(self):
         self.assertEqual(self.redmine.enumeration.resource_class.redmine_version, (2, 2, 0))
 
@@ -766,10 +849,10 @@ class StandardResourcesTestCase(BaseRedmineTestCase):
     @mock.patch('redminelib.open', mock.mock_open(), create=True)
     def test_attachment_download(self):
         response = responses['attachment']['get']
-        response['attachment']['content_url'] = 'http://foo/bar.txt'
+        response['attachment']['content_url'] = f'{self.url}/bar.txt'
         self.response.json.return_value = response
         self.response.iter_content = lambda chunk_size: (str(num) for num in range(0, 5))
-        self.assertEqual(self.redmine.attachment.get(1).download('/some/path'), '/some/path/bar.txt')
+        self.assertEqual(self.redmine.attachment.get(1).download('/some/path/'), '/some/path/bar.txt')
 
     def test_attachment_resource_map_converts_to_resource(self):
         self.response.json.return_value = responses['attachment']['get']
@@ -847,10 +930,10 @@ class StandardResourcesTestCase(BaseRedmineTestCase):
     @mock.patch('redminelib.open', mock.mock_open(), create=True)
     def test_file_download(self):
         response = responses['attachment']['get']
-        response['attachment']['content_url'] = 'http://foo/bar.txt'
+        response['attachment']['content_url'] = f'{self.url}/bar.txt'
         self.response.json.return_value = response
         self.response.iter_content = lambda chunk_size: (str(num) for num in range(0, 5))
-        self.assertEqual(self.redmine.file.get(1).download('/some/path'), '/some/path/bar.txt')
+        self.assertEqual(self.redmine.file.get(1).download('/some/path/'), '/some/path/bar.txt')
 
     def test_file_resource_map_converts_to_resource(self):
         self.response.json.return_value = responses['attachment']['get']
@@ -876,7 +959,7 @@ class StandardResourcesTestCase(BaseRedmineTestCase):
         wiki_page = self.redmine.wiki_page.get('Foo%Bar', project_id=1)
         self.assertEqual(self.patch_requests.call_args[0][1], f'{self.url}/projects/1/wiki/Foo%25Bar.json')
         self.assertEqual(wiki_page.title, 'Foo%Bar')
-        self.assertEqual(wiki_page.url, 'http://foo.bar/projects/1/wiki/Foo%25Bar')
+        self.assertEqual(wiki_page.url, f'{self.url}/projects/1/wiki/Foo%25Bar')
 
     def test_wiki_page_filter(self):
         self.response.json.return_value = responses['wiki_page']['filter']
@@ -965,7 +1048,7 @@ class StandardResourcesTestCase(BaseRedmineTestCase):
     def test_wiki_page_export(self):
         self.response.json.return_value = responses['wiki_page']['get']
         self.response.iter_content = lambda chunk_size: (str(num) for num in range(0, 5))
-        self.assertEqual(self.redmine.wiki_page.get('Foo', project_id='Foo').export('txt', '/foo'), '/foo/Foo.txt')
+        self.assertEqual(self.redmine.wiki_page.get('Foo', project_id='Foo').export('txt', '/foo/'), '/foo/Foo.txt')
 
     def test_wiki_page_parent_converts_to_resource(self):
         self.response.json.return_value = {'wiki_page': {'title': 'Foo', 'project_id': 1, 'parent': {'title': 'Bar'}}}
@@ -980,6 +1063,12 @@ class StandardResourcesTestCase(BaseRedmineTestCase):
         self.assertIsInstance(wiki_page.author, resources.User)
         self.assertEqual(wiki_page.author.firstname, 'John')
 
+    def test_wiki_page_sets_attrs_from_single_attr_id_map(self):
+        self.response.json.return_value = responses['wiki_page']['get']
+        wiki_page = self.redmine.wiki_page.get(1, project_id=1)
+        wiki_page.project_id = 1
+        self.assertEqual(wiki_page._decoded_attrs['project'], {'id': 1})
+
     def test_project_membership_version(self):
         self.assertEqual(self.redmine.project_membership.resource_class.redmine_version, (1, 4, 0))
 
@@ -1046,6 +1135,20 @@ class StandardResourcesTestCase(BaseRedmineTestCase):
         self.assertIsInstance(membership.group, resources.Group)
         self.assertEqual(membership.group.id, 1)
 
+    def test_project_membership_sets_attrs_from_single_attr_id_map(self):
+        self.response.json.return_value = responses['project_membership']['get']
+        membership = self.redmine.project_membership.get(1)
+        membership.project_id = 1
+        membership.user_id = 1
+        self.assertEqual(membership._decoded_attrs['project'], {'id': 1})
+        self.assertEqual(membership._decoded_attrs['user'], {'id': 1})
+
+    def test_project_membership_sets_attrs_from_multiple_attr_id_map(self):
+        self.response.json.return_value = responses['project_membership']['get']
+        membership = self.redmine.project_membership.get(1)
+        membership.role_ids = [1, 2]
+        self.assertEqual(membership._decoded_attrs['roles'], [{'id': 1}, {'id': 2}])
+
     def test_issue_category_version(self):
         self.assertEqual(self.redmine.issue_category.resource_class.redmine_version, (1, 3, 0))
 
@@ -1102,6 +1205,14 @@ class StandardResourcesTestCase(BaseRedmineTestCase):
         self.assertIsInstance(category.assigned_to, resources.User)
         self.assertEqual(category.assigned_to.firstname, 'John')
 
+    def test_issue_category_sets_attrs_from_single_attr_id_map(self):
+        self.response.json.return_value = responses['issue_category']['get']
+        category = self.redmine.issue_category.get(1)
+        category.project_id = 1
+        category.assigned_to_id = 1
+        self.assertEqual(category._decoded_attrs['project'], {'id': 1})
+        self.assertEqual(category._decoded_attrs['assigned_to'], {'id': 1})
+
     def test_issue_relation_version(self):
         self.assertEqual(self.redmine.issue_relation.resource_class.redmine_version, (1, 3, 0))
 
@@ -1147,6 +1258,12 @@ class StandardResourcesTestCase(BaseRedmineTestCase):
         self.response.json.return_value = responses['issue_relation']['get']
         self.assertEqual(self.redmine.issue_relation.get(1).url, f'{self.url}/relations/1')
 
+    def test_issue_relation_sets_attrs_from_single_attr_id_map(self):
+        self.response.json.return_value = responses['issue_relation']['get']
+        relation = self.redmine.issue_relation.get(1)
+        relation.issue_id = 1
+        self.assertEqual(relation._decoded_attrs['issue'], {'id': 1})
+
     def test_version_version(self):
         self.assertEqual(self.redmine.version.resource_class.redmine_version, (1, 3, 0))
 
@@ -1205,6 +1322,12 @@ class StandardResourcesTestCase(BaseRedmineTestCase):
         self.assertIsInstance(version.project, resources.Project)
         self.assertEqual(version.project.identifier, 'foo')
 
+    def test_version_relation_sets_attrs_from_single_attr_id_map(self):
+        self.response.json.return_value = responses['version']['get']
+        version = self.redmine.version.get(1)
+        version.project_id = 1
+        self.assertEqual(version._decoded_attrs['project'], {'id': 1})
+
     def test_user_version(self):
         self.assertEqual(self.redmine.user.resource_class.redmine_version, (1, 1, 0))
 
@@ -1228,6 +1351,15 @@ class StandardResourcesTestCase(BaseRedmineTestCase):
         self.assertEqual(users[1].id, 2)
         self.assertEqual(users[1].firstname, 'Jack')
 
+    def test_user_all_url_variations(self):
+        self.redmine.ver = (5, 0, 0)
+        self.assertEqual(self.redmine.user.all().manager.url, f'{self.url}/users.json?status=')
+        self.redmine.ver = (5, 1, 0)
+        self.assertEqual(self.redmine.user.all().manager.url, f'{self.url}/users.json?f[]=status_id&'
+                                            f'op[status_id]==&v[status_id][]=1&v[status_id][]=2&v[status_id][]=3')
+        self.redmine.ver = (6, 0, 0)
+        self.assertEqual(self.redmine.user.all().manager.url, f'{self.url}/users.json?status=*')
+
     def test_user_filter(self):
         self.response.json.return_value = responses['user']['filter']
         users = self.redmine.user.filter(status_id=2)
@@ -1286,6 +1418,8 @@ class StandardResourcesTestCase(BaseRedmineTestCase):
         self.response.json.return_value = responses['user']['get']
         user = self.redmine.user.get(1)
         self.assertIsInstance(user.issues, resultsets.ResourceSet)
+        self.assertIsInstance(user.issues_assigned, resultsets.ResourceSet)
+        self.assertIsInstance(user.issues_authored, resultsets.ResourceSet)
         self.assertIsInstance(user.time_entries, resultsets.ResourceSet)
 
     def test_user_includes(self):
@@ -1314,6 +1448,12 @@ class StandardResourcesTestCase(BaseRedmineTestCase):
         self.response.json.return_value = responses['user']['get']
         self.assertEqual(self.redmine.user.get(1).url, f'{self.url}/users/1')
 
+    @mock.patch('redminelib.open', mock.mock_open(), create=True)
+    def test_user_export(self):
+        self.response.json.return_value = responses['user']['all']
+        self.response.iter_content = lambda chunk_size: (str(num) for num in range(0, 5))
+        self.assertEqual(self.redmine.user.all().export('txt', '/foo/bar/'), '/foo/bar/users.txt')
+
     def test_group_version(self):
         self.assertEqual(self.redmine.group.resource_class.redmine_version, (2, 1, 0))
 
@@ -1383,6 +1523,12 @@ class StandardResourcesTestCase(BaseRedmineTestCase):
         self.response.json.return_value = responses['group']['get']
         self.assertEqual(self.redmine.group.get(1).url, f'{self.url}/groups/1')
 
+    def test_group_sets_attrs_from_multiple_attr_id_map(self):
+        self.response.json.return_value = responses['group']['get']
+        group = self.redmine.group.get(1)
+        group.user_ids = [1, 2]
+        self.assertEqual(group._decoded_attrs['users'], [{'id': 1}, {'id': 2}])
+
     def test_role_version(self):
         self.assertEqual(self.redmine.role.resource_class.redmine_version, (1, 4, 0))
 
@@ -1464,7 +1610,7 @@ class StandardResourcesTestCase(BaseRedmineTestCase):
     def test_news_export(self):
         self.response.json.return_value = responses['news']['all']
         self.response.iter_content = lambda chunk_size: (str(num) for num in range(0, 5))
-        self.assertEqual(self.redmine.news.all().export('txt', '/foo/bar'), '/foo/bar/news.txt')
+        self.assertEqual(self.redmine.news.all().export('txt', '/foo/bar/'), '/foo/bar/news.txt')
 
     def test_news_str(self):
         self.response.json.return_value = responses['news']['filter']
@@ -1495,6 +1641,12 @@ class StandardResourcesTestCase(BaseRedmineTestCase):
         self.response.json.return_value = response_includes
         self.assertIsInstance(news.comments, list)
 
+    def test_news_sets_attrs_from_single_attr_id_map(self):
+        self.response.json.return_value = responses['news']['get']
+        news = self.redmine.news.get(1)
+        news.project_id = 1
+        self.assertEqual(news._decoded_attrs['project'], {'id': 1})
+
     def test_issue_status_version(self):
         self.assertEqual(self.redmine.issue_status.resource_class.redmine_version, (1, 3, 0))
 
